function [F,S,f,Stopit]=edft(X,N,tk,I,W)

% EDFT Extended Discrete Fourier Transform.
%
% Function EDFT produce discrete N-point Fourier transform F and amplitude
% spectrum S of the data vector X. Data X may contain NaN (Not-a-Number).
%
% SYNTAX
%
% F=edft(X) or F=edft(X,[]) iteratively computes the Fourier transform of
% data X partially filled with NaN, otherwise Fast Fourier Transform (FFT)
% is applied, F=fft(X).
%
% F=edft(X,N) for length X less than N calculates N-point Fourier transform
%   iteratively. If N is vector then EDFT is computed at the query points
%   fn defined in N and N set to be equal length(fn). If the length of X
%   is greater than N, X is truncated to length N, and EDFT returns the
%   same result as fft(X,N) if there is no NaN in X.
%
% F=edft(X,N,tk) computes the EDFT of X using the sample points tk.
%   If input N is scalar then the EDFT query points calculated as
%   fn=ifftshift(-ceil((N-1)/2):floor((N-1)/2))/N. If tk is not specified
%   ([]) then tk=(0:length(X)-1). If the length of tk is greater than X, 
%   tk is truncated to the length X.
%
% F=edft(X,N,tk,I) performs the EDFT with limit I to the maximum number of
%   iterations. If not specified ([]), I default to value of parameter
%   'Miteration', edft(X,N)=edft(X,N,Miteration). To complete iteration
%   process faster the value for 'Miteration' should be decreased.
%
% F=edft(X,N,tk,I,W) executes the EDFT with initial conditions defined by
%   weight vector W. If not specified ([]) then W=ones(size(F)). W should
%   have at least length X nonzero elements for a stable output.
%
% [F,S]=edft(___) also computes amplitude spectrum S of data vector X. 
%   Vector S has the same size and phases as N-point Fourier transform 
%   vector F. S=F/length(X) in the special case if EDFT output F=fft(X)
%   or F=fft(X,N).
%
% [F,S,f]=edft(___) returns query points used by EDFT in f. 
%
% [F,S,f,Stopit]=edft(___) also returns informative parameter Stopit. 
% The first row of Stopit showing the number of performed iteration. 
% The second row indicate breaking of iteration reason and may have the
%   following values:
%  0 - Maximum number of iteration performed.
%  1 - Sum of outputs division, sum(F./S), is not equal to length(X)*N
%   within Relative deviation 'Rdeviat' or no unique solution found. The
%   calculations were interrupted because of results may be inaccurate.
%  2 - Relative threshold 'Rthresh' reached. To complete iteration process
%   faster the value for 'Rthresh' should be increased.
% Algorithm # used by EDFT is output in the third row of Stopit:
%  0 - A special case where FFT used
%  1 - Faster Algorithm for uniform data
%  2 - Algorithm with NaN in uniform data
%  3 - Algorithm for nonuniform data
%
% If X is a matrix, then EDFT treats columns of X as vectors and returns
%   the Fourier transform of each column. If N,W,tk are vectors, they are
%   applied to each column of X.
%
% ALGORITHM:
%
% Input:
%   X - input data.
%   N - length of discrete Fourier transform or number of query points in fn.
%   I - maximum number of iterations. If not specified, I=Miteration.
%   W - weight vector W. If not specified, W = ones(1,N) is used for the
%       first iteration.
%   E - Fourier transform basis matrix filled as E=exp(-i*2*pi*tk.'*fn).
%
% Output F, S for each EDFT iteration is calculated by following formulas:
%   R = E*diag(W/N)*E';
%   F = W.*(X*inv(R)*E);
%   S = (X*inv(R)*E)./diag(E'*inv(R)*E).';
%   W = S.*conj(S);	W used as input to the next EDFT iteration.
%   Algorithms 1 and 2 use functions FFT, IFFT and Levinson-Darbin
%   recursion to calculate R faster and speed up matrix multiplication.
%
% A special case:
%	If length of X is equal to N or length fn then EDFT output do not
%	depend on selected weight W and is calculated in non-iterative way.
%
% FEATURES:
%	1. EDFT output F is the N-point Fourier transform of data X. The Power
%   Spectral Density (PSD) function is calculated by the following
%   formula: abs(F).^2/(N*T), where T is mean sampling period.
%	2. EDFT output S is used to estimate amplitudes and phases of 
%   sinusoidal components in the data X.
%	3. EDFT can extrapolate X to the length N. For example, if apply EDFT
%   with N>length(X) get the result: F=edft(X,N)=edft(Y)=fft(Y), then 
%   Matlab function IFFT applied to the EDFT output F returns not only
%   input data undistorted but also extrapolates and interpolates the data
%   along the length of the transform, call line: Y=ifft(F), where Y is X
%   plus non-zero forward and backward extrapolation of X to the length N. 
%   4. If the data is nonuniform and/or frequencies fn are not in FFT grid
%   then the Matlab function NUFFT can be applied as the inverse Fourier
%   transform, call line: Y=nufft(F,f,-tk)/N;
%	5. EDFT can increase frequency resolution N/length(X) times. Division
%   of outputs 1/(T*F./S) demonstrate the frequency resolution of EDFT.
%   The following relationships are true for any EDFT iteration:
%       0<F./S<=N and sum(F./S)=N*length(X).
%	6. EDFT input X may contain NaN which indicate unavailable data or
%   missing samples or data segments. 
%
% TIPS for selection of EDFT inputs X(tk) and frequencies fn in N:
% 	1. Input sequence X(tk) for EDFT can be sampled uniformly or
%   nonuniformly. Uniform sampling can be considered as a special case of
%   nonuniform sampling, where tk=[0,1,...,K-1]*T. Nonuniform sampling can
%   be realized in many different ways, like as:
%	- uniform sampling with randomly missed samples (known as sparse data);
%	- uniform sampling with missed data segments (known as gapped data);
%	- uniform sampling with jitter: tk=([0,1,...,K-1] + jitter*rand(1,K))*T,
%       where value for jitter is selected in range [0...1[;
%	- additive nonuniform sampling: tk=tk-1 + (1+jitter*(rand-0.5))*T,
%       k=1,...,K-1, t0=0;
%	- signal dependent sampling, e.g, level-crossing sampling, etc... .
% 	2. Frequencies fn can be selected arbitrary. This mean, that user can
%   choose not only the length of EDFT (number of frequencies in fn) but
%   also the way how to distribute frequencies along the frequency axis.
%   On the other hand, in order to get an adequate representation of X in
%   the frequency domain, fn must cover the entire range in which it is 
%   supposed to find components, otherwise EDFT estimate will incorporate
%   power of X components that have spectra outside fn. The Fourier
%   analysis of X(tk) requires positive and negative frequency terms in fn.
%
% See also FFT, IFFT, NUFFT
%
% Author: Vilnis Liepins (vilnislp@gmail.com)
% References:
% 1) Vilnis Liepins, A spectral estimation method of nonuniformly sampled
%   band-limited signals. Automatic Control and Computer Sciences, Vol.28,
%   No.2, pp.66-77, 1994.
% 2) Vilnis Liepins, An algorithm for evaluation a discrete Fourier
%   transform for incomplete data. Automatic control and computer sciences,
%   Vol.30, No.3, pp.27-40, 1996.
% NOTE: The first version of file (gdft.m) was submitted to fileexchange 
%   on October 7, 1997 as Matlab 4.1 code.

%==========================Set default parameters==========================
Miteration=30;          % Limit for maximum number of iteration (Stopit 0)
Rdeviat=0.0001;         % Value for relative deviation (Stopit 1)
Rthresh=0.0001;         % Value for relative threshold (Stopit 2)
%=======================Setup input/output arguments=======================
if nargin==0||isempty(X), error('Not enough input arguments.'),end
% Input argument X
if sum(any(isinf(X))), error('Inf is not allowed in X.'),end
if size(X,1)==1, X=X.';trf=1;else,trf=0;end
[K, L]=size(X);
% Input argument N
fn=[];
if nargin<2||isempty(N)
    N=K;
elseif sum(any(isnan(N)))||sum(any(isinf(N)))
    error('NaN or Inf is not allowed in N.')
elseif isscalar(N), N=floor(abs(N));    % N is a scalar
elseif sum(any(isnan(X)))
    error('NaN is not allowed in X if N is a vector.')
else
    if size(N,1)==1, N=N(:);end         % N was vector row
    fn=real(N);[N, NL]=size(fn);        % Set frequencies fn
    if NL~=L&&NL~=1, error('Incorrect size of vector N (fn).')
    elseif NL==1&&L>1,fn=ones(N,L).*fn;end  % fn - 2 dim array
end
if N<K, X=X(1:N,:);K=N;end          % Truncate X if more than N points
% Input argument tk
if nargin>2&&~isempty(tk)
    if sum(any(isnan(X)))||sum(any(isnan(tk)))||sum(any(isinf(tk)))
        error('NaN or Inf is not allowed in X and tk.')
    elseif size(tk,1)==1,tk=tk(:);
    end
    [TK, TL] = size(tk);tk=real(tk);
    if TK>K, tk=tk(1:K,:);end       % Truncate tk if has more than X points
    if TK<K||(TL~=L&&TL~=1), error('Incorrect size of tk.')
    elseif TL==1&&L>1,tk=ones(K,L).*tk;end      % tk - 2 dim array
    if isempty(fn)
        fn=ones(N,L).*(ifftshift(-ceil((N-1)/2):floor((N-1)/2))/N).';
    end
elseif ~isempty(fn),tk=ones(K,L).*(0:K-1).';    % Use default tk
end
% Input argument I
if nargin<4||isempty(I), I=Miteration;  % Set default value for I
else
    I=floor(abs(I(1)));
    if isnan(I)||isinf(I), error('NaN and Inf is not allowed in I.'),end
end
% Input argument W
if nargin<5||isempty(W), W=ones(N,L);   % Set default values for W
elseif sum(any(isnan(W)))||sum(any(isinf(W)))
    error('NaN and Inf is not allowed in W.')
else
    if size(W,1)==1, W=W(:);end
    [WN, WL] = size(W);
    if WN~=N||(WL~=L&&WL~=1), error('Incorrect size of W.')
    elseif WL==1&&L>1,W=ones(N,L).*W;   % W - 2 dim array
    end
    W=W.*conj(W);WK=sum(W>eps);
    for l=1:L
        if WK(l)<K && WK(l)>0, W(:,l)=W(:,l)+max(W(:,l))*Rdeviat;  
        elseif WK(l)==0, W(:,l)=ones(N,1);end
    end
end
F=zeros(N,L);S=zeros(N,L);f=zeros(N,L);XR=zeros(K,1);RE=zeros(K,1);
ERE=zeros(N,1);Stopit=[I*ones(1,L);zeros(2,L);];SW=zeros(1,I);
%====================Process one by one each column of X===================
for l=1:L
    Xnan=~isnan(X(:,l));    % Xnan indicate samples as '1' , NaN as '0'
    KK=sum(Xnan);           % KK the length of input data X w/o NaN 
    if (~any(X(:,l))&&KK==K)||(KK==K&&K==N&&isempty(fn))||(KK==1&&K==1)||KK==0
        Alg=0;              % A special case - use FFT
        F(:,l)=fft(X(:,l),N);           
        S(:,l)=F(:,l)/K;
        Stopit(:,l)=[1; 0; Alg;];
    elseif isempty(fn)&&KK==K
        Alg=1;              % Algorithm without NaN in uniform sequence
        Stopit(3,l)=Alg;
        for it=1:I
           XR=zeros(K,1);RE=zeros(K,1);Alg1_ERE;
            if stopit1,break,end    % Break if Rdeviat reached
            F(:,l)=fft(XR,N);       % Calculate EDFT output
            calc_edft_out;
            if stopit2,break,end    % Break if Rthresh reached
        end
    elseif KK<K
        Alg=2;              % Algorithm with NaN in uniform sequence
        Stopit(3,l)=Alg;
        X(~Xnan,l)=zeros(K-KK,1);   % Replace NaN by 0 in X
        t=find(Xnan);               % Sample number vector
        INVR=zeros(K);ER=zeros(K,1);  
        for it=1:I
            Alg2_ERE;
            if stopit1,break,end    % Break if Rdeviat reached
            F(:,l)=fft(conj(INVR)*X(:,l),N); % Calculate EDFT output
            calc_edft_out;
            if stopit2,break,end    % Break if Rthresh reached
        end    
    else 
        Alg=3;  % Algorithm for nonuniform/uniform sequence/frequency set
        if K==N,I=1;W=ones(N,l);end
        Stopit(:,l)=[I; 0; Alg;];
        E=exp(-1i*2*pi*tk(:,l)*fn(:,l).');  % Complex exponents matrix E
        for it=1:I
            Alg3_ERE;
            if stopit1,break,end    % Break if Rdeviat reached
            F(:,l)=X(:,l).'*RE;     % Calculate EDFT output
            calc_edft_out;
            if stopit2,break,end    % Break if Rthresh reached
        end
    end
    if Alg~=3,f(:,l)=ifftshift(-ceil((N-1)/2):floor((N-1)/2)).'/N;
    else,f(:,l)=fn(:,l);end
end                                 % All columns of X processed
Stopit=table(Stopit,'RowNames',{'Iteration #';'Break Reason';'Algorithm #'});
if trf==1,F=F.';S=S.';f=f.';end     % Adjust size of EDFT output
%=============================Nested functions=============================
    function st1=stopit1
    stit=abs(ERE.'*W(:,l)/N/KK-1);st1=false;
        if (stit>Rdeviat&&it~=1)||isnan(stit)
            Stopit(1:2,l)=[it-1; 1;];st1=true;
        end
    end
    function st2=stopit2 
        SW(it)=sum(W(:,l));st2=false;
        if it>1
            thit=abs(SW(it-1)-SW(it))/SW(1);
            if thit<=Rthresh
                Stopit(1:2,l)=[it; 2;];st2=true;
            end
        end
    end
    function calc_edft_out
        S(:,l)=F(:,l)./ERE;
        F(:,l)=F(:,l).*W(:,l);
        W(:,l)=S(:,l).*conj(S(:,l));
    end
    function Alg1_ERE
        r=ifft(W(:,l));
        [a,V]=levinson(r,K-1);
        a=a.';
        XR=zeros(K,1);RE=zeros(K,1);rc=a;
        for k=1:K/2
        k0=K-k+1;k1=2:K-2*k+1;k2=k+1:K-k;k3=k:K-k+1;
        RE(1)=RE(1)+2*rc(k);
        RE(k0-k+1)=RE(k0-k+1)+2*rc(k0);
        RE(k1)=RE(k1)+4*rc(k2);
        XR(k)=XR(k)+rc(k3)'*X(k3,l);
        XR(k0)=XR(k0)+(flipud(rc(k3))).'*X(k3,l);
        XR(k2)=XR(k2)+rc(k2)*X(k,l)+flipud(conj(rc(k2)))*X(k0,l);
        rc(k2)=rc(k2-1)+conj(a(k+1))*a(k2)-a(k0)*flipud(conj(a(k2+1)));
        end
        if mod(K,2)==1
            RE(1)=RE(1)+rc(k+1);XR(k+1)=XR(k+1)+X(k+1,l)*rc(k+1);
        end
        ERE=real(fft(RE,N));W(:,l)=W(:,l)/real(V);
    end
    function Alg2_ERE
        R=Alg2_R;     
        if rcond(R(t,t))>eps
            INVR(t,t)=R(t,t)\eye(KK);
        else
            W(:,l)=W(:,l)+Rdeviat*max(W(:,l));
            R=Alg2_R;
            INVR(t,t)=pinv(R(t,t));
        end   
        ER(1)=trace(INVR);
        for k=1:K-1
            ER(k+1,1)=sum(diag(INVR,k)+conj(diag(INVR,-k)));
        end
        ERE=real(fft(ER,N));       
        function R=Alg2_R
            RT=ifft(W(:,l));
            R=toeplitz(RT(1:K));
        end
    end
    function Alg3_ERE
        R=Alg3_R;
        if rcond(R)>eps
            RE=R\E;
        else 
            W(:,l)=W(:,l)+Rdeviat*max(W(:,l));
            R=Alg3_R;
            RE=pinv(R)*E;
        end
        ERE=sum(conj(E).*RE).';
        function R=Alg3_R
            R=E*diag(W(:,l)/N)*E';
        end
    end
end