% This program calculates and plots output of spectral estimation methods 
% discussed in [5]:
% - Discrete Fourier Transform (DFT)  - Matlab FFT and NUFFT programs used;
% - Extended DFT (EDFT);
% - High-Resolution DFT (HRDFT) [3];
% EDFT spectral estimate is tested on five K=64-point sequences:
% 1) x_uni - uniform complex-value test sequence taken from [4];
% 2) x_non - nonuniform complex-value test sequence taken from [4];
% 3) x_spa - sparse nonuniform real-value sequence [2];
% 4) x_mak - well known Marple&Kay 64-point real-value sequence [1];
% 5) x_wgn - real-value white Gaussian noise sequence.
%
% References:
% [1] S.M.Kay, S.L.Marple. Spectrum analysis - a modern perspective. 
% Proceedings IEEE 69, 1981.
% [2] V.Liepins. An algorithm for evaluation a discrete Fourier 
% transform for incomplete data, Automatic control and computer sciences, 
% Vol.30, No.3, pp.27-40, 1996.
% [3] Mauricio D. Sacchi, Tadeusz J. Ulrych, Colin J. Walker. Interpolation 
% and Extrapolation Using a High-Resolution Discrete Fourier Transform. 
% IEEE Trans. on Signal Proc., 46(1), 1998.
% [4] Vilnis Liepins. High-resolution spectral analysis by using basis 
% function adaptation approach. Doctoral Thesis for Scientific Degree of 
% Dr.Sc.Comp., University of Latvia, 1997.
% [5] Vilnis Liepins. Extended Fourier analysis of signals. 
% arXiv:1303.2033, 2013.
%
% Author: Vilnis Liepins (vilnislp@gmail.com)

clear
rng(5883,"twister");
it=15;					    % Number of EDFT and HRDFT iterations
N=1000;					    % Number of frequencies
fn=(-ceil((N-1)/2):floor((N-1)/2))/N; % Uniform frequency set [-0.5...0.5[
fne=(-ceil((2*N-1)/2):floor((2*N-1)/2))/N;% Extended frequency set [-1...1[
fnn=fn+0.8*(rand(1,N)-0.5)/N; % Nonuniform frequency set [-0.5...0.5[
fnr=(0:floor((N-1)/2))/N;   % Positive frequency set [0...0.5[
K=64;                       % Length of test sequences 1, 2, 3 and 5
t_u=0:K-1;                  % Uniform time set (sampling period=1)
t_n=t_u+rand(1,K)*0.8;      % Nonuniform time set (mean sampling period=1)

%=====================================================================
% SEQUENCE 1: x_uni - Uniform K-point complex-value sequence
%=====================================================================
PHT1=2*pi*(rand-0.5);	% Random initial phase of the complex exponent 1
PHT2=2*pi*(rand-0.5);	% Random initial phase of the complex exponent 2
freq1=0.35;				% Normalized frequency of the complex exponent 1
freq2=0.3985;			% Normalized frequency of the complex exponent 2
x_s1=exp(1i*(2*pi*freq1*t_u+PHT1));	% Complex exponent 1 
x_s2=exp(1i*(2*pi*freq2*t_u+PHT2));	% Complex exponent 2
x_i=20*sin(pi*(t_u-K/2)*0.25)./(pi*(t_u-K/2)+eps).*exp(1i*pi*0.25*(t_u-K/2));
if round(K/2)==K/2, x_i(K/2+1)=5;end	% Rectangular pulse [0...0.25]
f_x=-0.5:1/1024:-0.25;  % Simulate a band-limited noise as sum of sinusoids
rand_ph=2*rand(257,1)*ones(1,K);
x_n=sum(exp(1i*2*pi*(f_x'*t_u+rand_ph)))/sqrt(K); % Band-limited noise [-0.5...-0.25]
x_uni=x_s1+x_s2+x_i+x_n;    % x_uni - uniform composite sequence
xmax=ceil(max(abs(x_uni)));
lim=2^9+0.5;
sigm_x=x_uni*x_uni'/K;
% Simulate 10-bit Analog-to-Digital Converter
x_uni=xmax*round(x_uni/xmax*lim)/lim;	
sigm_n=sqrt((xmax/lim)^2/6/K); 
sigm_p=(xmax/lim)^2*6/N;
% WT, WP WE - true spectra
WT=[20*ones(1,N/4)/K sigm_n*ones(1,N/4) 20*ones(1,N/4)/K sigm_n*ones(1,N/4)];
WP=[(20*ones(1,N/4)).^2/N sigm_p*ones(1,N/4) (20*ones(1,N/4)).^2/N sigm_p*ones(1,N/4)];
if min(abs(fn-freq1))<K/N/N
[~, wt_i]=min(abs(fn-freq1));WT(wt_i)=1;WP(wt_i)=N;
else
[~, wt_i]=min(abs(fn-freq1+1/2/N));WT(wt_i)=1;WP(wt_i)=N;
[~, wt_i]=min(abs(fn-freq1-1/2/N));WT(wt_i)=1;WP(wt_i)=N;
end
if min(abs(fn-freq2))<K/N/N
[wt_m, wt_i]=min(abs(fn-freq2));WT(wt_i)=1;WP(wt_i)=N;
else
[~, wt_i]=min(abs(fn-freq2+1/2/N));WT(wt_i)=1;WP(wt_i)=N;
[wt_m, wt_i]=min(abs(fn-freq2-1/2/N));WT(wt_i)=1;WP(wt_i)=N;
end
WE=[sigm_n*ones(1,N/2) WT sigm_n*ones(1,N/2)];

%=====================================================================
% SEQUENCE 1 x_uni with NaNs inserted:
% x_u48 - sequence x_uni with 1/4 skipped samples
% x_u40 - sequence x_uni with 3/8 skipped samples
%=====================================================================
% Generate skipped sample numbers for sequences x_u48, x_u40
x_u48=x_uni;x_u40=x_uni;x_48=x_uni;x_40=x_uni;
f2k=floor(K/2);k32=(1:2:f2k*2)+round(rand(1,f2k));
k24=k32(round(1:4/3:f2k));
k16=k32(1:2:f2k);
% Insert zeros (FFT input) or NaNs (EDFT input) where samples are skipped on random basis
x_48(k16)=zeros(1,length(k16));	% x_48 - x_uni with 1/4 (16) zeros (FFT input)
x_u48(k16)=NaN*ones(1,length(k16));	% x_u48 -  x_uni with 1/4 (16) NaNs (EDFT input)
x_40(k24)=zeros(1,length(k24));	% x_40 -  x_uni with 3/8 (24) zeros (FFT input)
x_u40(k24)=NaN*ones(1,length(k24));	% x_u40 -  x_uni with 3/8 (24) NaNs (EDFT input)

%=====================================================================
% SEQUENCE 2: x_non - Nonuniform K-point complex-value sequence
%=====================================================================
x_ns1=exp(1i*(2*pi*freq1*t_n+PHT1));	% Complex exponent 1 at frequency freq1
x_ns2=exp(1i*(2*pi*freq2*t_n+PHT1));	% Complex exponent 2 at frequency freq2
% Pulse [0 ... 0.25]
x_ni=20*sin(pi*(t_n-K/2)*0.25)./(pi*(t_n-K/2)).*exp(1i*pi*0.25*(t_n-K/2));
% Band-limited noise [-0.5 ...-0.25]
x_nn=sum(exp(1i*2*pi*(f_x'*t_n+rand_ph)))/sqrt(K); 
x_non=x_ns1+x_ns2+x_ni+x_nn;		% x_non - nonuniform composite test sequence
% Simulate 10-bit Analog-to-Digital Converter
x_non=xmax*round(x_non/xmax*lim)/lim; 

%=====================================================================
% SEQUENCE 3: x_spa - Real-value K-point sequence on random 1 sec grid
%=====================================================================
% x_spa consisting of num_fr sinusoids with different amplitudes and random phases. 
% White Gaussian noise, SNR~20 [dB]. Frequncies of sinusoids are on DFT grid 1/N.
num_am=[0.5 1 2 3];	        % Amplitudes of sinusoinds in the x_spa
SNR_s=20;					% Signal to noise ratio [dB]
num_fr=length(num_am);		% Number of sinusoids in the x_spa
x_spa=NaN*ones(1,N);		% Prefill with NaN (EDFT input)
x_spa0=zeros(1,N);  		% Prefill with zeros (FFT input)
t_ran=randi(floor(N/K)-1,1,64);
t_spa=floor(linspace(0,N-N/K,K))+t_ran;	% Sparse time for x_spa
fr_spa=floor((N-1)*rand(1,num_fr)/2)/N;% Generate frequencies on grid [0 ... 0.5[
ph_spa=2*pi*rand(1,num_fr);	% Random phases of sinusoids
s_spa=zeros(1,K);
for k=1:num_fr
s_spa=s_spa+num_am(k)*sin(2*pi*fr_spa(k)*t_spa+ph_spa(k));	% Generate signal part
end
sigma=sqrt((sum(num_am.^2))/10^(SNR_s/10));	% Sigma for given SNR
n_spa=sigma*randn(1,K)/sqrt(2);	% Generate noise part 
SNR_c=round(10*log10((s_spa*s_spa.')/(n_spa*n_spa.')));	% Calculated SNR
x_spa(t_spa+1)=s_spa+n_spa;		% Sequence with noise (EDFT input)
x_spa0(t_spa+1)=s_spa+n_spa;	% Sequence with noise (FFT input)

%=====================================================================
% SEQUENCE 4: Marple&Kay 64-point data set x_mak 
%=====================================================================
x_mak=[1.291061 -2.086368 -1.691316 1.243138 1.641872 -0.008688 -1.659390 -1.111467 ...
0.985908 1.991979 -0.046613 -1.649269 -1.040818 1.054665 1.855816 -0.951182 -1.476495 ...
-0.212242 0.780202 1.416003 0.199282 -2.027026 -0.483577 1.664913 0.614114 -0.791469 ...
-1.195311 0.119801 0.807635 0.895236 -0.012734 -1.763842 0.309840 1.212892 -0.119905 ...
-0.441686 -0.879733 0.306181 0.795431 0.189598 -0.342332 -0.328700 0.197881 0.071179 ...
0.185931 -0.324595 -0.366092 0.368467 -0.191935 0.519116 0.008328 -0.425946 0.651478 ...
-0.639978 -0.344389 0.814130 -0.385168 0.064218 -0.380008 -0.163008 1.180961 0.114206 ...
-0.667626 -0.814997];
KK=length(x_mak);

%=====================================================================
% SEQUENCE 5: White Gaussian noise sequence x_wgn - K uniform samples	
%=====================================================================
x_wgn=randn(1,K);
%=====================================================================

disp('Calculating DFT output...');	
dft_x_uni=fftshift(fft(x_uni,N))/K;
dft_x_ue=nufft(x_uni,t_u,fne)/K;
dft_x_non=nufft(x_non,t_n,fn)/K;
dft_x_31=nufft(x_uni,t_u,fnn)/K;
dft_x_33=nufft(x_non,t_n,fnn)/K;
dft_x_ne=nufft(x_non,t_n,fne)/K;
fft_xmak=fft(x_mak,N);
fft_xwgn=fft(x_wgn,N);
dft_x_mak=fft_xmak/KK;
dft_x_48=fftshift(fft(x_48,N))/length(x_48);
dft_x_40=fftshift(fft(x_40,N))/length(x_40);
dft_xspa1=fft(x_spa0,N)/K;

disp('Calculating Extended DFT (EDFT) output...');
[F1a,S1a]=edft(x_uni,N,[],1,fftshift(WT));
[F1,S1]=edft(x_uni,N,[],it);
S1a=fftshift(S1a);S1b=fftshift(S1);F1b=fftshift(F1);
[F2,S2]=edft(x_non,fn,t_n,it);
[F2a,S2a]=edft(x_non,fn,t_n,1,WT);
[F31,S31]=edft(x_uni,fnn,[],it);
[F33,S33]=edft(x_non,fnn,t_n,it);
[F4a,S4a]=edft(x_uni,fne,t_u,it);
[F4b,S4b]=edft(x_non,fne,t_n,it);
[F5b,S5b]=edft(x_u48,N,[],it);
[F5c,S5c]=edft(x_u40,N,[],it);
[F6b,S6b]=edft(x_spa(t_spa+1),fn,t_spa,it);
F6b=fftshift(F6b);S6b=fftshift(S6b);
[F8b,S7b]=edft(x_mak,N,[],it);
F9b=edft(x_wgn,N,[],it);

% Calculate HRDFTs
disp('Calculating High-Resolution DFT (HRDFT) output...');
W=ones(1,N);			% Initial conditions W for Marple&Kay sequence
W1=ones(1,N);			% Initial conditions W1 for white Gaussian noise
for l=1:it
fh78=edft(x_mak,N,[],1,W);		% Perform one EDFT iteration with weight W
fh9=edft(x_wgn,N,[],1,W1);		% Perform one EDFT iteration with weight W1
W=fh78/N;		            % Calculate weights for the next
W1=fh9/N;		            % HRDFT iteration
end 

% Calculate plots on Figures 1-9
%======== Figure 1 ========
true_sp=20*log10(WT);		% True Power spectrum of test sequences 1 and 2
true_psd=10*log10(WP);		% True PSD of test sequences 1 and 2	
dft_f1=20*log10(abs(dft_x_uni));
edft_f1a=20*log10(abs(S1a));
edft_f1b=20*log10(abs(S1b));
dft_f1c=10*log10(abs(K*dft_x_uni).^2/N);
edft_f1c=10*log10(abs(F1b).^2/N);
edft_f1d=real(F1b./S1b/K);
%======== Figure 2 ========
dft_f2=20*log10(abs(dft_x_non));
edft_f2a=20*log10(abs(S2a));
edft_f2b=20*log10(abs(S2));
dft_f2c=10*log10(abs(K*dft_x_non).^2/N);
edft_f2c=10*log10(abs(F2).^2/N);
edft_f2d=real(F2./S2/K);
%======== Figure 3 ========
dft_f31=10*log10(abs(K*dft_x_31).^2/N);
edft_f31=10*log10(abs(F31).^2/N);
dft_f33=10*log10(abs(K*dft_x_33).^2/N);
edft_f33=10*log10(abs(F33).^2/N);
edft_f32=real(F31./S31/K);
edft_f34=real(F33./S33/K);
%======== Figure 4 ========
dft_f4a=20*log10(abs(dft_x_ue));
edft_f4a=20*log10(abs(S4a));
dft_f4b=20*log10(abs(dft_x_ne));
edft_f4b=20*log10(abs(S4b));
edft_f4c=real(F4b./S4b/K/2);
%======== Figure 5 ========
dft_f5b=20*log10(abs(dft_x_48));
edft_f5b=20*log10(abs(fftshift(S5b)));
dft_f5c=20*log10(abs(dft_x_40));
edft_f5c=20*log10(abs(fftshift(S5c)));
%======== Figure 6 ========
dft_f6b=[abs(dft_xspa1(1)) 2*abs(dft_xspa1(2:floor(N/2)))];
edft_f6b=[abs(S6b(1)) 2*abs(S6b(2:floor(N/2)))];
edft_f6c=real(F6b(1:floor(N/2))./S6b(1:floor(N/2))/N);
edft_f6d=real(ifft(F6b));
%======== Figure 7 ========
dft_f7a=20*log10([abs(dft_x_mak(1)) 2*abs(dft_x_mak(2:floor(N/2)))]);
edft_f7b=20*log10([abs(S7b(1)) 2*abs(S7b(2:floor(N/2)))]);
hrf_f7c=20*log10([abs(fh78(1)/N) 2*abs(fh78(2:floor(N/2))/N)]+eps);
%======== Figure 8 ========
ifft_f8a=real(ifft(fft_xmak));
iedft_f8b=real(ifft(F8b));
ihrdft_f8c=real(ifft(fh78));
%======== Figure 9 ========
ifft_f9a=real(ifft(fft_xwgn));
iedft_f9b=real(ifft(F9b));
ihrdft_f9c=real(ifft(fh9));

disp('Plot figures...')
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 1: True spectrum, DFT and EDFT estimates for SEQUENCE 1 - x_uni
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++            
figure(1)
clf
db_min=-80;db_max=10;		% Axis min and max in [dB]
subplot(411)
plot(fn,true_sp,'r-',fn,dft_f1,'b-',fn,edft_f1a,'k-') % Plot True, DFT, non-iterative EDFT
axis([-0.5 0.5 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title(['UNIFORM sequence: (a) - True Power Spectrum [red], DFT [blue]' ...
    ' and non-iterative EDFT'])
subplot(412)
plot(fn,true_sp,'r-',fn,dft_f1,'b-',fn,edft_f1b,'k-') % Plot True, DFT and EDFT
axis([-0.5 0.5 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title(['(b) - True Power Spectrum [red], DFT [blue] and EDFT ' ...
    ,int2str(it),'th iteration'])
subplot(413)
plot(fn,true_psd,'r-',fn,dft_f1c,'b-',fn,edft_f1c,'k-') % Plot True, DFT and EDFT
axis([-0.5 0.5 db_min db_max+30])
xlabel('Frequency')
ylabel('PSD [dB]')
title(['(c) - True PSD [red], DFT [blue] and EDFT ' ...
    '',int2str(it),'th iteration'])
subplot(414)
plot(fn,ones(1,N),'b-',fn,edft_f1d,'k-')	% Plot DFT and EDFT relative resolution
axis([-0.5 0.5 0 ceil(N/K)])
xlabel('Frequency')
ylabel('Resolution')
title(['(d) - Relative frequency resolution of DFT [blue] and EDFT' ...
    ' ',int2str(it),'th iteration'])

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 2: True spectrum, DFT and EDFT estimates for SEQUENCE 2 - x_non           
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure(2)
clf
subplot(411)
plot(fn,true_sp,'r-',fn,dft_f2,'b-',fn,edft_f2a,'k-')% Plot True, DFT, non-iterative EDFT
axis([-0.5 0.5 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title(['NONUNIFORM sequence: (a) - True Power Spectrum [red], ' ...
    'DFT [blue] and non-iterative EDFT'])
subplot(412)
plot(fn,true_sp,'r-',fn,dft_f2,'b-',fn,edft_f2b,'k-');% Plot spectrum- True, DFT, EDFT
axis([-0.5 0.5 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title(['(b) - True Power Spectrum [red], DFT [blue] and EDFT' ...
    ' ',int2str(it),'th iteration'])
subplot(413)
plot(fn,true_psd,'r-',fn,dft_f2c,'b-',fn,edft_f2c,'k-');% Plot PSD - True, DFT, EDFT
axis([-0.5 0.5 db_min db_max+30])
xlabel('Frequency')
ylabel('PSD [dB]')
title(['(c) - True PSD [red], DFT [blue] and EDFT ',int2str(it),'th iteration'])
subplot(414)
plot(fn,ones(1,N),'b-',fn,edft_f2d,'k-')	% Plot DFT and EDFT relative resolution
axis([-0.5 0.5 0 ceil(N/K)])
xlabel('Frequency')
ylabel('Resolution')
title(['(d) - Relative frequency resolution of DFT [blue] and EDFT ' ...
    '',int2str(it),'th iteration'])

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 3: True PSD, DFT and EDFT estimates on non-uniform frequency grid           
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure(3)
clf
subplot(411)
plot(fn,true_psd,'r-',fnn,dft_f31,'b-',fnn,edft_f31,'k-')% Plot PSD - True, DFT, EDFT
axis([-0.5 0.5 db_min db_max+30])
xlabel('Frequency')
ylabel('PSD [dB]')
title(['NONUNIFORM frequency grid and UNIFORM sequence: (a) - True PSD [red],' ...
    ' DFT [blue] and EDFT'])
subplot(412)
plot(fnn,ones(1,N),'b-',fnn,edft_f32,'k-')	% Plot DFT and EDFT relative resolution
axis([-0.5 0.5 0 ceil(N/K)])
xlabel('Frequency')
ylabel('Resolution')
title(['(b) - Relative frequency resolution of DFT [blue] and EDFT' ...
    ' ',int2str(it),'th iteration'])
subplot(413)
plot(fn,true_psd,'r-',fnn,dft_f33,'b-',fnn,edft_f33,'k-')% Plot PSD - True, DFT, EDFT
axis([-0.5 0.5 db_min db_max+30])
xlabel('Frequency')
ylabel('PSD [dB]')
title(['NONUNIFORM frequency grid and NONUNIFORM sequence: (c) - True PSD [red],' ...
    ' DFT [blue] and EDFT'])
subplot(414)
plot(fnn,ones(1,N),'b-',fnn,edft_f34,'k-')	% Plot DFT and EDFT relative resolution
axis([-0.5 0.5 0 ceil(N/K)])
xlabel('Frequency')
ylabel('Resolution')
title(['(d) - Relative frequency resolution of DFT [blue] and EDFT' ...
    ' ',int2str(it),'th iteration'])

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 4: True spectrum, DFT and EDFT estimates in extended frequency range             
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure(4)
clf
true_e=20*log10(WE);	% Extended True spectrum of test sequences 1 and 2
subplot(311)
plot(fne,true_e,'r-',fne,dft_f4a,'b-',fne,edft_f4a,'k-') % Spectrums- True, DFT, EDFT
axis([-1 1 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title('UNIFORM sequence: (a) - True Power Spectrum [red], DFT [blue] and EDFT')
subplot(312)
plot(fne,true_e,'r-',fne,dft_f4b,'b-',fne,edft_f4b,'k-') % Spectrums- True, DFT, EDFT
axis([-1 1 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title('NONUNIFORM sequence: (b) - True Power Spectrum [red], DFT [blue] and EDFT')
subplot(313)
plot([-0.5 0;-0.25 0.25],[1 1;1 1],'r:');
axis([-1 1 0 ceil(N/K)])
hold on
line([freq1; freq1],[N/K; 0],'linestyle',':','color','r')
line([freq2; freq2],[N/K; 0],'linestyle',':','color','r')
plot(fne,ones(1,2*N)/2,'b-',fne,edft_f4c,'k-')	% Plot DFT and EDFT relative resolution
hold off
xlabel('Frequency')
ylabel('Resolution')
title(['NONUNIFORM sequence: (c) - Relative frequency resolution of DFT' ...
    ' [blue] and EDFT'])

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 5: True spectrum, DFT and EDFT estimates for SEQUENCE 1 with NaNs inserted
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure(5)
clf
subplot(311)
plot(fn,true_sp,'r-',fn,dft_f1,'b-',fn,edft_f1b,'k-') % Plot True, DFT and EDFT for sequence w/o NaN
axis([-0.5 0.5 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title(['True Power Spectrum [red], DFT [blue] and EDFT. UNIFORM sequence:' ...
    ' (a) - 64 samples'])
subplot(312)
plot(fn,true_sp,'r-',fn,dft_f5b,'b-',fn,edft_f5b,'k-')	% Spectrum of sequence with 16 NaNs 
axis([-0.5 0.5 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title('UNIFORM sequence: (b) - 16 skipped samples' )
subplot(313)
plot(fn,true_sp,'r-',fn,dft_f5c,'b-',fn,edft_f5c,'k-')	% Spectrum of sequence with 24 NaNs
axis([-0.5 0.5 db_min db_max])
xlabel('Frequency')
ylabel('Power [dB]')
title('UNIFORM sequence: (c) - 24 skipped samples')

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 6: SEQUENCE 3 - x_spa, and DFT, EDFT estimates and reconstructed data
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure(6)
clf
subplot(411)
m_xspa=max(abs(x_spa0))+1;
stem(1:N,x_spa,'ko');				% Plot 64-point sequence
hold on
plot(1:N,x_spa,'ro');
hold off
axis([0 N -m_xspa m_xspa])
xlabel('Sample number')
ylabel('Magnitude')
title(['(a) Nonuniform real-value sequence - ',int2str(K),' samples, SNR=' ...
    '',int2str(SNR_c), ' dB'])
subplot(412)
stem(fr_spa,num_am,'ro');		% Prot true amplitudes
axis([0 0.5 0 max(num_am)+0.5])
hold on
plot(fnr,dft_f6b,'b-',fnr,edft_f6b,'k-')	% Plot estimates by DFT and EDFT
hold off
xlabel('Frequency')
ylabel('Amplitude')
title('(b) True amplitudes [o] and estimates by DFT [blue] and EDFT')
subplot(413)
% Plot DFT and EDFT relative resolution
plot(fnr,ones(1,floor(N/2))/N*K,'b-',fnr,edft_f6c,'k-')	
axis([0 0.5 0 1])
xlabel('Frequency')
ylabel('Resolution')
title('(c) Relative frequency resolution of DFT [blue] and EDFT')
subplot(414)
% Plot 1000-point intrepolated (uniform) sequence by EDFT
plot(1:N,edft_f6d,'k-',1:N,x_spa,'ro');	
axis([0 N -m_xspa m_xspa])
xlabel('Sample number')
ylabel('Magnitude')
title(['(d) Original [o] and interpolated sequence by Inverse EDFT - ' ...
    '',int2str(N),' samples'])

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 7: True spectrum, DFT, EDFT and HRDFT estimates of SEQUENCE 4 - x_mak
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure(7)
clf
subplot(311)
fr_true=[0.1 0.2 0.21; 0.1 0.2 0.21;]; % True spectrum for Marple&Kay test sequence
amp_true=[-20 0 0; db_min db_min db_min];
col_n=15*log10(0.1*(sin(pi*(eps:0.001:1))).^2);	% Colored noise as described in [1]
plot(0.2:0.0003:0.5,col_n,'-r')
axis([0 0.5 db_min db_max])
hold on 
line(fr_true,amp_true,'linestyle','-','color','r')
plot(fnr,dft_f7a,'k-')			% Plot DFT Power Spectrum
hold off
xlabel('Frequency')
ylabel('Power [dB]')
title('Marple&Kay sequence: (a) - True Power Spectrum [red] and DFT')
subplot(312)
plot(0.2:0.0003:0.5,col_n,'-r')
axis([0 0.5 db_min db_max])
hold on
line(fr_true,amp_true,'linestyle','-','color','r')
plot(fnr,edft_f7b,'k-')		% Plot EDFT Power Spectrum
hold off
xlabel('Frequency')
ylabel('Power [dB]')
title(['(b) - True Power Spectrum [red] and EDFT ',int2str(it),'th iteration'])
subplot(313)
plot(0.2:0.0003:0.5,col_n,'-r')
axis([0 0.5 db_min db_max])
hold on  
line(fr_true,amp_true,'linestyle','-','color','r')
plot(fnr,hrf_f7c,'k-')			% Plot HRDFT Power Spectrum
hold off
xlabel('Frequency')
ylabel('Power [dB]')
title(['(c) - True Power Spectrum [red] and HRDFT ',int2str(it),'th iteration'])

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 8: SEQUENCE 4 - x_mak and reconstructed data by DFT, EDFT and HRDFT
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure(8)
clf
subplot(311)
x_m6=max(abs(ihrdft_f8c));
plot(ifft_f8a,'k-')			% Plot Inverse DFT sequence
axis([0 N+1 -x_m6 x_m6])
hold on
plot(x_mak,'b-')			% Plot Marple&Kay sequence
hold off
xlabel('Sample number')
ylabel('Magnitude')
title('Marple&Kay sequence: (a) - True (blue) and reconstructed by Inverse DFT')
subplot(312)
plot(iedft_f8b,'k-')			% Plot Inverse EDFT sequence
axis([0 N+1 -x_m6 x_m6])
hold on
plot(x_mak,'b-')			% Plot Marple&Kay sequence
hold off
xlabel('Sample number')
ylabel('Magnitude')
title('(b) - True (blue) and reconstructed by Inverse EDFT')
subplot(313)
plot(ihrdft_f8c,'k-')			% Plot Inverse HRDFT sequence
axis([0 N+1 -x_m6 x_m6])
hold on
plot(x_mak,'b-')			% Plot Marple&Kay sequence
hold off
xlabel('Sample number')
ylabel('Magnitude')
title('(c) - True (blue) and reconstructed by Inverse HRDFT')

%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Figure 9: SEQUENCE 5 - x_wgn and reconstructed data by DFT, EDFT and HRDFT
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
figure(9)
clf
subplot(311)
x_m7=max(abs(ihrdft_f9c));
plot(1:N,ifft_f9a,'k-')			% Plot Inverse DFT sequence
axis([0 N+1 -x_m7 x_m7])
hold on
plot(1:K,x_wgn,'b-')			% Plot white Gaussian noise sequence
axis([0 N+1 -x_m7 x_m7])
hold off
xlabel('Sample number')
ylabel('Magnitude')
title(['White Gaussian noise sequence: (a) - True (blue) and reconstructed' ...
    ' by Inverse DFT'])
subplot(312)
plot(iedft_f9b,'k-')			% Plot Inverse EDFT sequence
axis([0 N+1 -x_m7 x_m7])
hold on
plot(x_wgn,'b-')			% Plot white Gaussian noise sequence
hold off
xlabel('Sample number')
ylabel('Magnitude')
title('(b) - True (blue) and reconstructed by Inverse EDFT')
subplot(313)
plot(ihrdft_f9c,'k-')			% Plot Inverse HRDFT sequence
axis([0 N+1 -x_m7 x_m7])
hold on
plot(x_wgn,'b-')			% Plot white Gaussian noise sequence
hold off
xlabel('Sample number')
ylabel('Magnitude')
title('(c) - True (blue) and reconstructed by Inverse HRDFT')